@@ -18,4 +18,8 @@ from .TopkBMMCuda import TopkBMMCuda
 
 from .IVFPQTopkCuda import IVFPQTopkCuda
 from .IVFPQTop1Cuda import IVFPQTop1Cuda
-from .TopkSelectCuda import TopkSelectCuda
\ No newline at end of file
+from .DistributedIVFPQTopkCuda import DistributedIVFPQTopkCuda
+from .DistributedIVFPQTop1Cuda import DistributedIVFPQTop1Cuda
+from .TopkSelectCuda import TopkSelectCuda
+from .Top32SelectCuda import Top32SelectCuda
+from .Top1SelectCuda import Top1SelectCuda
\ No newline at end of file
